/*
    Copyright (C) 1999 by Joe Pasqua <joe@NoRoomAtTheInn.org>.
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.


    THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;

class DateType extends Type {

    DateType() {
        methods = new Hashtable(1);
        methods.put("compare", new CompareMethod());
    }

    static class CompareMethod implements Type.Method {

        public short[] codeGen(Value target, Vector inputs)
                throws ParseException
        {
            Arg[]  argDesc = {
                new Arg("IsCond", "relational operator"),
                new Arg("Date", "rhs")
            };
            Vector  args = Arg.getArgs(argDesc, inputs);

            return Type.encode(
                0x01, (0x60 + ((IsCondVal)args.elementAt(0)).condition()),
                0, daysSinceEpoch((DateVal)args.elementAt(1)));
        }

        int daysSinceEpoch(DateVal d) {
            Date  epoch = new Date("12/31/1998");
            Date  date = new Date(d.year-1900, d.month-1, d.date);
            long eTime = epoch.getTime();
            long diff = date.getTime() - eTime;
            diff /= (1000 * 60 * 60 * 24);
            return (int)diff;
        }

    }
}

