/*
    Copyright (C) 1999 by Joe Pasqua <joe@NoRoomAtTheInn.org>.
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.


    THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

import java.util.Hashtable;
import java.util.Vector;


class PointType extends Type {

    PointType() {
        methods = new Hashtable(1);
        methods.put("turnOn", new TurnOnMethod());
        methods.put("turnOff", new TurnOffMethod());
        methods.put("isOff", new StateMethod(0));
        methods.put("isOn", new StateMethod(1));
        methods.put("turnsOff", new StateMethod(2));
        methods.put("turnsOn", new StateMethod(3));
    }

    class TurnOnMethod implements Type.Method {

        public short[] codeGen(Value target, Vector inputs)
                throws ParseException
        {
            Arg.getArgs(null, inputs);  // Make sure there are no args

            return Type.encode(
                0x04, 0x00, ((PointVal)target).getEncoded(), 1);
        }
    }

    class TurnOffMethod implements Type.Method {

        public short[] codeGen(Value target, Vector inputs)
                throws ParseException
        {
            Arg.getArgs(null, inputs);  // Make sure there are no args

            return Type.encode(
                0x04, 0x00, ((PointVal)target).getEncoded(), 0);
        }
    }

    class StateMethod implements Type.Method {

        int val;

        StateMethod(int val) { this.val = val; }

        public short[] codeGen(Value target, Vector inputs)
                throws ParseException
        {
            Vector  args = Arg.getArgs(null, inputs);

            return Type.encode(
                0x00, 0x00, ((PointVal)target).getEncoded(), val);
        }
    }

}

