/*
    Copyright (C) 1999 by Joe Pasqua <joe@NoRoomAtTheInn.org>.
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.


    THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

class Symbol {
    String  name;
    Type    type;
    Value   initVal;
    boolean isVar;

    Symbol(String name, Type type, BasicVal bv, boolean isVar)
            throws ParseException
    {
        this.name = name;
        this.type = type;
        this.isVar = isVar;

        if ((type.equals(Type.get("Number")) || type.equals(Type.get("Timer")))
                && bv == null)  // Must be a var if bv == null
            bv = new BasicVal("0");
        this.initVal = (bv == null) ? null : type._new(bv, isVar);
    }

    Value getValue() { return initVal; }

    public String toString() {
        StringBuffer buf = new StringBuffer(80);
        buf.append(type);
        buf.append(" ");
        buf.append(name);
        if (initVal != null) {
            buf.append(" = new ");
            buf.append(type);
            buf.append('(');
            buf.append(initVal);
            buf.append(");");
        }
        return buf.toString();
    }

    static class Variable extends Symbol {
        Variable(String name, Type type)
            throws ParseException
        {
            super(name, type, null, true);
        }

        Variable(String name, Type type, BasicVal bv)
            throws ParseException
        {
            super(name, type, bv, true);
        }
    }

    static class Constant extends Symbol {
        Constant(String name, Type type, BasicVal bv)
            throws ParseException
        {
            super(name, type, bv, false);
        }
    }

}


